/**
 * @author Chenzhyc
 * @description CC视频播放器
 */

 import React, { Component } from 'react';
 import PropTypes from 'prop-types';
 import appendQuery from 'append-query';
 
 const ccPlayerHost = 'https://p.bokecc.com/player';
 const playerType = '1';
 function getSWF(swfID) {
	 if (window.document[swfID]) {
		 return window.document[swfID];
	 } else if (navigator.appName.indexOf("Microsoft") == -1) {
		 if (document.embeds && document.embeds[swfID]) {
			 return document.embeds[swfID];
		 }
	 } else {
		 return document.getElementById(swfID);
	 }
 }
 class PBUVideoPlayer extends Component {
	 constructor(props) {
		 super(props);
		 this.state = {
			 ...props
		 };
		 this.videoBox = null
		 this.intervalTimer = null;
		 this.startTime = 0;
		 this.currentTimer = null; //获取实时播放器进度的定时器
	 }
 
	 playVideo = () => {
		 if (window.cc_js_Player.getVideoState() == 'pause') {
			 window.cc_js_Player.play()
		 }
	 }
 
	 pasueVideo = () => {
		 if (window.cc_js_Player.getVideoState() == 'play') {
			 window.cc_js_Player.pause()
		 }
	 }
 
	 jumpToTime = (second) => {
		 window.cc_js_Player.jumpToTime(second)
	 }
 
	 getPosition = () => {
		 return window.cc_js_Player.getPosition()
	 }
 
	 getCurrRate = () => {
		 return window.cc_js_Player.getCurrRate()
	 }
 
	 // 关闭进度计时器
	 onCloseListenCurrent = () => {
		 if (this.currentTimer) {
			 clearInterval(this.currentTimer)
		 }
	 }
 
	 // 每秒回调当前播放进度(关联倍速)
	 onListenCurrent = () => {
		 const interval = 1 / this.getCurrRate()
		 this.currentTimer = setInterval(() => {
			 const time = Math.floor(window.cc_js_Player.getPosition())
			 this.props.onRuntimeChange(time)
		 }, interval * 1000);
	 }
 
	 onRecordPlaytime = (duration, currentTime) => {
		 this.props.onRecordPlaytime(duration, currentTime)
	 }
 
	 openTimer() {
		 if (!this.props.onRecordPlaytime) {
			 return
		 }
		 this.startTime = Date.now()
		 const interval = this.props.interval
		 this.intervalTimer = setInterval(() => {
			 this.startTime = Date.now()
			 this.onRecordPlaytime(interval)
		 }, interval * 1000)
	 }
 
	 // 计算本次播放暂时的时长
	 getDuration() {
		 if (!this.props.onRecordPlaytime) {
			 return
		 }
		 clearInterval(this.intervalTimer)
		 const currentTime = Date.now()
		 const duration = Math.ceil((currentTime - this.startTime) / 1000)
		 this.startTime = currentTime
		 if (duration == 1) {
			 return
		 }
		 this.onRecordPlaytime(duration, Math.floor(this.getPosition()))
	 }
 
	 loadScript() {
		 //先删掉之前的视频播放器节点
		 if (this.videoBox.hasChildNodes()) {
			 this.videoBox.removeChild(this.videoBox.childNodes[0]);
		 }
		 window.clearInterval(this.intervalTime);
 
		 const videoSrc = appendQuery(ccPlayerHost, {
			 vid: this.state.vid,
			 siteid: this.state.siteId,
			 autoStart: this.state.autoStart,
			 width: this.state.width,
			 height: this.state.height,
			 playerid: this.state.playerid,
			 playertype: playerType
		 });
 
		 const oScript = document.createElement('script');
		 oScript.type = 'text/javascript';
		 oScript.async = true;
		 oScript.src = videoSrc;
		 this.videoBox.appendChild(oScript);
	 }
 
	 componentDidMount() {
 
		 this.loadScript();
 
		 window.on_CCH5player_ready = (obj) => {
			 console.log('CCH5player_ready', obj);
		 }
 
		 window.on_CCH5player_play = () => {
			 this.openTimer()
			 if (this.props.onRuntimeChange) {
				 this.onListenCurrent()
			 }
			 console.log('2222',this.props.isCanPlayState)
			 if (!this.props.isCanPlayState) {
				 this.pasueVideo()
			 }
			 console.log('CCH5player_play');
		 }
 
		 window.on_CCH5player_pause = () => {
			 this.onCloseListenCurrent()
			 this.getDuration()
			 console.log('CCH5player_pause');
		 }
 
		 window.on_CCH5player_ended = () => {
			 this.onCloseListenCurrent()
			 this.getDuration()
			 console.log('CCH5player_ended');
		 }
		 window.get_cc_verification_code = (vid) => {
			 return this.props.onGetVerificationCode(vid);
		 }
 
		 window.on_switch_rate = (vid, newRate) => {
			 clearInterval(this.currentTimer)
			 this.onListenCurrent()
		 }
 
 
		 window.on_spark_player_start = () => {
			 this.openTimer()
			 console.log('on_spark_player_start');
		 }
 
		 window.on_spark_player_pause = () => {
			 this.getDuration()
			 console.log('on_spark_player_pause');
		 }
 
		 window.on_spark_player_resume = () => {
			 this.openTimer()
			 console.log('on_spark_player_resume');
		 }
 
		 window.on_spark_player_stop = () => {
			 this.getDuration()
			 console.log('on_spark_player_stop');
		 }
 
		 window.on_spark_player_ready = () => {
			 console.log('on_spark_player_ready');
		 }
 
		 window.on_player_seek = (prevTime, currentTime) => {
			 this.props.onPlayerSeek(prevTime, currentTime)
			 console.log('on_player_seek');
 
		 }
 
		 window.on_cc_player_init = (vid, objectId) => {
			 this.player = getSWF(objectId);
			 const config = {
				 on_player_stop: "on_spark_player_stop",
				 on_player_pause: "on_spark_player_pause",
				 on_player_resume: "on_spark_player_resume",
				 on_player_ready: "on_spark_player_ready",
				 on_player_start: "on_spark_player_start",
				 on_player_seek: "on_spark_player_seek"
			 }
 
			 this.player.setConfig(config);
		 }
	 }
 
	 componentWillReceiveProps(nextProps) {
		 this.setState({
			 ...nextProps
		 });
	 }
 
	 shouldComponentUpdate(nextProps, nextState) {
		 return nextProps.vid !== this.props.vid;
	 }
 
	 componentDidUpdate() {
		 this.startTime = 0;
		 this.loadScript();
	 }
 
	 componentWillUnmount() {
		 if (window.cc_js_Player.getVideoState() == 'play') {
			 this.getDuration()
		 }
		 clearInterval(this.intervalTimer);
	 }
 
	 render() {
		 return (
			 <div ref={node => this.videoBox = node}></div>
		 )
	 }
 }
 
 PBUVideoPlayer.propTypes = {
	 /**
		* siteId
		*/
	 siteId: PropTypes.string.isRequired,
 
	 /**
		* 视频唯一标识符
		*/
	 vid: PropTypes.string.isRequired,
	 /**
		* 播放器宽度
		*/
	 width: PropTypes.string,
	 /**
		* 播放器高度
		*/
	 height: PropTypes.string,
	 /**
		* 是否自动播放，默认false
		*/
	 autoStart: PropTypes.bool,
	 /**
		* 播放器id，
		*/
	 playerid: PropTypes.string,
	 /**
		* @param currentPosition  已播放的秒数
		*/
	 onRecordPlaytime: PropTypes.func,
	 /**
		* 获取用于鉴权的验证码
		* @param {string} vid 视频id
		* @return {string}
		*/
	 onGetVerificationCode: PropTypes.func,
	 /**
			* 定时回调播放时长
			* @param {number}  间隔时间
			*/
	 interval: PropTypes.number,
	 // 每秒回调视频播放进度
	 onRuntimeChange: PropTypes.func,
	 // 拖拽进度条的回调
	 onPlayerSeek: PropTypes.func,
	 // 当前是否允许播放
	 isCanPlayState: PropTypes.bool
 }
 
 PBUVideoPlayer.defaultProps = {
	 vid: '',
	 siteId: '',
	 width: 600,
	 height: 490,
	 autoStart: false,
	 playerid: '',
	 interval: 60,
	 onGetVerificationCode: vid => vid,
	 onPlayerSeek: () => { },
	 isCanPlayState:true,
 }
 
 
 export default PBUVideoPlayer;
 